#ifndef LIB_KYAUDIOREG_H
#define LIB_KYAUDIOREG_H


#define KYAUDIOREG_MAJOR 0
#define KYAUDIOREG_MINOR 1
#define KYAUDIOREG_PATCH 0


#include <stdbool.h>
#include <stdint.h>

typedef enum {
    KY_AUDIO_REG_LOG_LEVEL_ERROR,
    KY_AUDIO_REG_LOG_LEVEL_WARN,
    KY_AUDIO_REG_LOG_LEVEL_INFO,
    KY_AUDIO_REG_LOG_LEVEL_DEBUG,
    KY_AUDIO_REG_LOG_LEVEL_TRACE,
} KyAudioRegLogLevel;

typedef enum {
    KY_AUDIO_REG_LOG_LEVEL_FILTER_OFF,
    KY_AUDIO_REG_LOG_LEVEL_FILTER_ERROR,
    KY_AUDIO_REG_LOG_LEVEL_FILTER_WARN,
    KY_AUDIO_REG_LOG_LEVEL_FILTER_INFO,
    KY_AUDIO_REG_LOG_LEVEL_FILTER_DEBUG,
    KY_AUDIO_REG_LOG_LEVEL_FILTER_TRACE,
} KyAudioRegLogLevelFilter;

typedef struct KyAudioRegulator KyAudioRegulator;

typedef void (*KyAudioRegLogCb)(KyAudioRegLogLevel level,
                                const char *file,
                                unsigned int line,
                                const char *target,
                                const char *msg,
                                void *userdata);

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

bool kyaudioreg_init_logger(KyAudioRegLogLevelFilter max_level,
                            KyAudioRegLogCb log_cb,
                            void *userdata);

KyAudioRegulator *kyaudioreg_new(uint32_t sample_rate,
                                 uint32_t sample_size,
                                 uint32_t alloc_size,
                                 uint32_t target_buffering);

void kyaudioreg_delete(KyAudioRegulator *audioreg);

void kyaudioreg_push(KyAudioRegulator *audioreg,
                     uint32_t missing_samples,
                     const uint8_t *src,
                     size_t len);

void kyaudioreg_pull(KyAudioRegulator *audioreg, uint8_t *dst, size_t len);

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus

#endif /* LIB_KYAUDIOREG_H */
