#ifndef VLC_AUDIO_REGULATOR_H
#define VLC_AUDIO_REGULATOR_H

#include <vlc_common.h>

#include <stdint.h>

#include <vlc_objects.h>
#include <vlc_frame.h>

struct vlc_audio_regulator {
    struct vlc_object_t obj;

    uint32_t sample_rate;
    size_t sample_size;
    uint32_t target_buffering;

    const struct vlc_audio_regulator_ops *ops;
    void *sys;
};

struct vlc_audio_regulator_ops {
    void (*push)(struct vlc_audio_regulator *ar, uint32_t missing_samples,
                 const void *data, size_t len_bytes);

    void (*pull)(struct vlc_audio_regulator *ar, void *data, size_t len_bytes);

    void (*close)(struct vlc_audio_regulator *ar);
};

typedef int
vlc_audio_regulator_open_fn(struct vlc_audio_regulator *ar);

#define set_callback_audio_regulator(open) { \
    vlc_audio_regulator_open_fn *fn = open; \
    (void) fn; \
    set_callback(fn); \
}

VLC_API struct vlc_audio_regulator *
vlc_audio_regulator_New(vlc_object_t *obj, const char *module_name,
                        uint32_t sample_rate, size_t sample_size,
                        uint32_t target_buffering);

VLC_API void
vlc_audio_regulator_Delete(struct vlc_audio_regulator *ar);

static inline void
vlc_audio_regulator_Push(struct vlc_audio_regulator *ar,
                         uint32_t missing_samples, const void *data,
                         size_t len_bytes)
{
    ar->ops->push(ar, missing_samples, data, len_bytes);
}

static inline void
vlc_audio_regulator_PushBlock(struct vlc_audio_regulator *ar, block_t *block) {
    vlc_audio_regulator_Push(ar, block->missing_audio_samples, block->p_buffer,
                             block->i_buffer);
}

static inline void
vlc_audio_regulator_Pull(struct vlc_audio_regulator *ar, void *data,
                         size_t len_bytes)
{
    ar->ops->pull(ar, data, len_bytes);
}

#endif
